#!/usr/bin/perl -w

###
# Project:     pflogstats
# Module:      pflogstats-extensions-addressmapping.pm
# Description: Address mapping module
# Copyright:   Dr. Peter Bieringer <pbieringer at aerasec dot de>
#               AERAsec GmbH <http://www.aerasec.de/> 
# License:     GNU GPL v2
# CVS:         $Id: pflogstats-extensions-addressmapping.pm,v 1.12 2003/07/01 07:59:51 rootadm Exp $
###

## ChangeLog
#	0.01:
#	 - initial
#	0.02:
#	 - add support of regexp
#	0.03:
#	 - don't modify special bounce addresses
#       0.04
#        - make Perl 5.0 compatible
#	0.05
#	 - display table contents only in verbose mode
##

use strict;


## Local constants
my $module_type = "extensions";
my $module_name = $module_type . "-addressmapping";
my $module_version = "0.05";

## Export module info
$main::moduleinfo{$module_name}->{'version'} = $module_version;
$main::moduleinfo{$module_name}->{'type'} = $module_type;
$main::moduleinfo{$module_name}->{'name'} = $module_name;

## Global prototyping
sub beforemainloopstarts();
sub modifyaddress($);
sub help();


## Local prototyping
sub read_address_mapping($);


## Register options
$main::options{'addressmapping_file=s'} = \$main::opts{'addressmapping_file'};


## Register calling hooks
$main::hooks{'beforemainloopstarts'}->{$module_name} = \&beforemainloopstarts;
$main::hooks{'modifyaddress'}->{$module_name} = \&modifyaddress;
$main::hooks{'help'}->{$module_name} = \&help;


## Local variables
my %address_mapping;
my %address_mapping_evalpattern;


## Global callable functions

# Help
sub help() {
	my $helpstring = "
    Extension: addressmapping
    [--addressmapping_file <file>]  maps addresses into another addresses
                                     format: 2 colums
                                      regular expressions are also supported
";
	return $helpstring;
};

# Initialization
sub beforemainloopstarts() {
	# print "Called: " . $module_name . "/" . "beforemainloopstarts" . "\n";

	# Read file contents
	if (defined $main::opts{'addressmapping_file'} ) {
		&read_address_mapping($main::opts{'addressmapping_file'});
	};
};

# Modify an address depeding on mapping
sub modifyaddress($) {
	my $address = shift || die "ERROR: arg1 (address) missing";

	# Do not modify special bounce addresses
	if ($address eq "from=<>" || $address eq "from=<#@[]>") {
		return $address;
	};

	# Simple replacement
	if (defined $address_mapping{$address} ) {
		# Match, so replace
		$address = $address_mapping{$address};
		return $address;
	};

	# regex support
	print STDERR "Check against pattern: " . $address . "\n" if ($main::opts{'debug'} & 0x1000);
	for my $pattern ( keys %address_mapping ) {
		print STDERR "Pattern: " . $pattern . "\n" if ($main::opts{'debug'} & 0x1000);
		if ( $address =~ /^$pattern$/ ) {
			#print STDERR "Patternmatch: " . $address . "\n" if ($main::opts{'debug'} & 0x1000);
			$address = eval $address_mapping_evalpattern{$pattern};
			if (! defined $address) { $address = "EMPTY_ADDRESS"; };
			print STDERR "Replaced: " . $address . "\n" if ($main::opts{'debug'} & 0x1000);
			return $address;
		};
	};
	
	# Return address
	return $address;
};


## Local functions

# Read address mapping file
sub read_address_mapping($) {
	my $filename = shift || die "ERROR: arg1 (filename) missing";
	my ($address_found, $address_replacement);

	# open file
	open(ADDRESSMAPPING,"<" . $filename) || die "ERROR: cannot open: " . $filename;

	# read file contents
	while(<ADDRESSMAPPING>) {
		#printf STDERR $_ if ($opts{'debug'} & 0x0040);
		$_ =~ s/([^#]*)#.*/$1/; # Remove comments
                $_ =~ s/[[:space:][:cntrl:]]+$//; # Remove trailing spaces and control chars
                $_ =~ s/^[[:space:][:cntrl:]]+//; # Remove leading spaces and control chars
                if ( length($_) == 0 ) { next; }; # Skip empty lines
	
		# Split
		($address_found, $address_replacement) = split " ", $_, 2;

		if ( (defined $address_found) && (defined $address_replacement) ) {
			if ( (length($address_found) > 0) && (length($address_replacement) > 0) ) {
				if (! defined $address_mapping{$address_found} ) {
					$address_mapping{$address_found} = $address_replacement;

					# Generate eval pattern
					$address_replacement =~ s/@/\\@/;
					$address_replacement = "return \"" . $address_replacement . "\"";
					$address_mapping_evalpattern{$address_found} = $address_replacement;
				} else {
					warn "WARNING: 1st value already found and defined, therefore skipped in " . $filename;
				};
			} else {
				warn "WARNING: line isn't valid and therefore skipped in " . $filename;
			};
		} else {
			warn "WARNING: line isn't valid and therefore skipped in " . $filename;
		};
	};

	close(ADDRESSMAPPING);

	# Print contents (debug)
        if ( scalar (keys %address_mapping) > 0 ) {
		if (defined $main::opts{'verbose'}) {
			print '='x79 . "\n";
			print "Address mapping table\n";
			print '-'x79 . "\n";
			for $address_found (keys %address_mapping) {
				print $address_found . " -> " . $address_mapping{$address_found} . "\n";
			};
			print '='x79 . "\n";
		};
        };
};



## End of module
return 1;
